// Code.java                                    MM 2008
//                                              AZ 2005

import java.io.*;                       // Datenstreams

/**
 * Code als Bytefolge mit Dateizugriff
 */

public class Code
{
/* ------------------------------------------------- */
                                            // coreData
/**
 * Code als Bytefolge.
 */
  private byte[] bytes;

/**
 * Name des Codedatei.
 */
  private String dateiName;

/* ------------------------------------------------- */
                                       // Konstruktoren
/**
 * Kostruktor.
 */
  public Code()
  {
    bytes = null;
    dateiName = null;
  }

/**
 * Kostruktor, uebernimmt Code als Bytefolge.
 * @param bytes Code als Folge von Bytes
 */
  public Code( byte[] bytes)
  {
    this();
                                               // Kopie
    int n = bytes.length;
    this.bytes = new byte[ n];
    System.arraycopy( bytes, 0, this.bytes, 0, n);
  }

/**
 * Kostruktor, liest Code als Bytefolge aus Datei.
 * @param dateiName Dateiname der Codedatei
 */
  public Code( String dateiName)
    throws Exception
  {
    this();
    leseAusDatei( dateiName);
  }

/* ------------------------------------------------- */
                                     //getData-Methoden
/**
 * Gibt Dateinamen zurueck.
 */
  public String getDateiName()
  {
    return dateiName;
  }

/**
 * Bestimmen der Codelaenge in Bytes.
 */
  public int getAnzahlBytes()
  {
    if( bytes == null) return 0;
    return bytes.length;
  }

/**
 * Gibt i. Byte zurueck.
 * @param i Index
 */
  public byte getByteAt( int i)
  {
    return bytes[ i];
  }

/* ------------------------------------------------- */
                                         //Dateizugriff
/**
 * Liest Code aus Datei mit angegebenen Dateinamen.
 * @param dateiName Dateiname der Codedatei
 */
  public void leseAusDatei( String dateiName)
    throws Exception
  {
    this.dateiName = dateiName;
    leseAusDatei();
  }

/**
 * Liest Code aus Datei.
 */
  private void leseAusDatei()
    throws Exception
  {
    bytes = new byte
            [(int)(( new File( dateiName)).length())];

    FileInputStream in
    = new FileInputStream( dateiName);

    in.read( bytes);
    in.close();
  }

/**
 * Schreibt Code in Datei mit angegebenen Dateinamen.
 * @param dateiName Dateiname der Codedatei
 */
  public void schreibeInDatei( String dateiName)
    throws Exception
  {
    this.dateiName = dateiName;
    schreibeInDatei();
  }

/**
 * Schreibt Code in Datei.
 */
  private void schreibeInDatei()
    throws Exception
  {
    FileOutputStream out
    = new FileOutputStream( dateiName);

    if( bytes != null) out.write( bytes);
    out.close();
  }

/* ------------------------------------------------- */
/**
 * Darstellen des Codes als String.
 */
  public String toString()
  {
    if( bytes == null) return "";
    return new String( bytes);
  }

/* ------------------------------------------------- */

/**
 * Test der Klasse Code,
 * Kopiert Code aus einer Datei in eine andere,
 * Aufruf: java Code <Quelldatei> <Zieldatei>.
 */
  public static void main( String[] args)
  {
    try
    {
// Ueberschrift
      System.out.println();
      System.out.println
      ( "Kopie: " + args[ 0] + " -> " + args[ 1] + "\n");
      System.out.println();

// Kopie
      Code code = new Code( args[ 0]);         // Lesen
      System.out.println( code);            // Anzeigen
      code.schreibeInDatei( args[ 1]);     // Schreiben

      System.out.println( "Fertig!");
    }

// Kommandozeile unvollstaendig
    catch( ArrayIndexOutOfBoundsException e)
    {
      System.err.print( "Aufruf: ");
      System.err.println
      ( "java Code <Quelldatei> <Zieldatei>");
    }

// Datei existiert nicht
    catch( FileNotFoundException e)
    {
      System.err.println
      ( "Die Datei >" + args[ 0] + "< existiert nicht!");
    }

// Fehlerbehandlung allgemein
    catch( Exception e)
    {
      System.err.println( "Fehler: " + e);
    }
  }
}