// XORCodierer.java                             MM 2008
//                                              AZ 2005

import java.io.*;                       // Datenstreams

/**
 * XOR - Codierer,
 * verknuepft byteweise Code und Schluessel mit XOR (^).
 */

public class XORCodierer
  extends Codierer
{
/**
 * Schluessel, mit dem der XOR-Codierer arbeitet.
 */
  private int schluessel;

/* ------------------------------------------------- */
/**
 * Konstruktor, setzt Code und Schluessel.
 * @param code zu verschluesselnde Code
 * @param schluessel verwendeter Schluessel
 */
  public XORCodierer( Code code, int schluessel)
  {
    super( code);
    this.schluessel = schluessel;
  }

/* ------------------------------------------------- */
/**
 * Codieren mit XOR (^),
 * ueberschreibt abstrakte Codierer-Methode.
 */
  public Code codieren()
  {
    int n = getCode().getAnzahlBytes();
    byte[] buffer = new byte[ n];

    if( n > 0)
      for( int i = 0; i < n; i++)
      {
        buffer[ i]
        = (byte)( getCode().getByteAt( i) ^ schluessel);
      }

    Code neuCode = new Code( buffer);
    return neuCode;
  }

/* ------------------------------------------------- */

/**
 * Test der Klasse XORCodierer.
 * Verschluesselt und entschluesselt "Auto"
 * mit dem Schluessel 16.
 */
  public static void main( String[] args)
  {
// Original erzeugen
    byte auto[] = { 'A', 'u', 't', 'o'};
    Code original = new Code( auto);

// Schluessel
    int schluessel = 16;

// Verschluesseln
    XORCodierer codierer
    = new XORCodierer( original, schluessel);
    Code codiert = codierer.codieren();

// Entschluesseln
    XORCodierer decodierer
    = new XORCodierer( codiert, schluessel);
    Code decodiert = decodierer.codieren();

// Code anzeigen
    System.out.println( "Original:       " + original);
    System.out.println( "Codiert:        " + codiert);
    System.out.println( "Entschluesselt: " + decodiert);

    System.out.println( "Fertig!");
  }
}
