// XORController.java                           MM 2005
// MVC-Controller                               AZ 2005

import java.awt.event.*;    // Adapter, Listener, Event
import javax.swing.*;                         //JSlider
import javax.swing.event.*; // JSlider-Listener, -Event
import java.awt.*;                        // FileDialog

/**
 * Controller zum XORView
 */

public class XORController
  extends WindowAdapter
  implements ActionListener, ChangeListener
{
/**
 * Mathematisches Modell,
 * enthaelt Funktionalitaet dedes XOR-Codierers
 */
  private XORModel model;

/**
 * Zum Controller gehoeriger View.
 */
  private XORView view;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor
 * @param model Model, welches dargestellt werden soll
 * @param view  zum Controller gehoeriger View
 */
  public XORController( XORModel model, XORView view)
  {
    this.model = model;
    this.view = view;
  }

/* ------------------------------------------------- */
                                // handleEvent-Methoden
/**
 * ActionListener,
 * Ereignisverarbeitung Munuepunkte.
 */
  public void actionPerformed( ActionEvent ae)
  {
    String command = ae.getActionCommand();

    if( command.equals( view.ACTION_NEU))
    {
      view.reset();
      model.reset();
    }

    if( command.equals( view.ACTION_CODIEREN))
    {
      model.codieren( view.getSlKey());
    }

// Dateizugriffe
    if( command.equals( view.ACTION_ORIGINAL))
    {
      String dateiName
      = whichDateiName( command, true);
      try
      {
        if( dateiName != null)
        {
          view.setMiCodieren( true);
          model.liesAusDatei( dateiName);
        }
      }
      catch( Exception e)
      {
        setFehlerDialog
        ( e.toString(), "Dateifehler " + command);
        view.reset();
      }
    }

    if( command.equals( view.ACTION_SPEICHERN))
    {
      String dateiName
      = whichDateiName( command, false);
      try
      {
        if( dateiName != null)
          model.schreibeInDatei( dateiName);
      }
      catch( Exception e)
      {
        setFehlerDialog
        ( e.toString(), "Dateifehler " + command);
      }
    }

    if( command.equals( view.ACTION_EXIT))
    {
      view.release();
    }
  }

/**
 * Laden des Standarddialogs zum
 * Oeffnen/Speichern von Dateien.
 * @param titel Fenstertitel
 * @param load  true, Laden einer Datei
 *              false, Speichern einer Datei
 * @return Dateiname 
 */
  private String whichDateiName
  ( String titel, boolean load)
  {
    int which = load ? FileDialog.LOAD
                     : FileDialog.SAVE;

    FileDialog dialog
    = new FileDialog( view, titel, which);

    dialog.setVisible( true);
    return dialog.getFile();
  }

/**
 * Abfangen von Dateifehlern.
 * @param fehlerMeldung Fehlermeldung
 * @param titel Fenstertitel
 */
  public void setFehlerDialog
  ( String fehlerMeldung, String titel)
  {
    JOptionPane.showMessageDialog( view, fehlerMeldung,
      titel, JOptionPane.ERROR_MESSAGE);
  }

/* ------------------------------------------------- */

/**
 * ChangeListener,
 * Ereignisverarbeitung Schieberegister.
 */
  public void stateChanged( ChangeEvent ce)
  {
    JSlider slider =(JSlider)ce.getSource();
    view.setLbKey( slider.getValue());
  }

/* ------------------------------------------------- */

/**
 * WindowsListener,
 * Ereignisverarbeitung Schliessen des Fensters.
 */
  public void windowClosing( WindowEvent we)
  {
    view.release();
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Freigabe des Controllers,
 * setzt Model und View zurueck.
 */
  public void release()
  {
    model = null;
    view = null;
  }
}