// XORView.java                                 MM 2008
// MVC-View                                     AZ 2005

import java.awt.event.*;                    // Mnemonic
import javax.swing.*;                // GUI-Komponenten
import java.util.*;                         // Observer

/**
 * View zum XORModel.
 */

public class XORView
  extends JFrame
  implements Observer
{
/* ------------------------------------------------- */
                                      // GUI-Konstanten
                                               // Menue
  public  final String ACTION_NEU = "Neu";
  private final int ACTION_KEY_NEU = KeyEvent.VK_N;

  public  final String ACTION_ORIGINAL = "Original laden";
  private final int ACTION_KEY_ORIGINAL = KeyEvent.VK_O;

  public  final String ACTION_CODIEREN = "Codieren";
  private final int ACTION_KEY_CODIEREN = KeyEvent.VK_C;

  public  final String ACTION_SPEICHERN = "Speichern unter";
  private final int ACTION_KEY_SPEICHERN = KeyEvent.VK_S;

  public  final String ACTION_EXIT = "Beenden";
  private final int ACTION_KEY_EXIT = KeyEvent.VK_B;

                                     // Schieberegister
  private final int KEY_MIN = 0;
  private final int KEY_MAX = 256;
  private final int KEY_DEF = 0;

/* ------------------------------------------------- */
                                     // GUI-Komponenten
/**
 * Originalcode.
 */
  private JTextArea input;

/**
 * Label fuer Dateinamen Originalcode.
 */
  private JLabel lbInput;

/**
 * Verschluesselter Code.
 */
  private JTextArea output;

/**
 * Label fuer Dateinamen Verschluesselter Code.
 */
  private JLabel lbOutput;

/**
 * Schieberegister Schluessel.
 */
  private JSlider slKey;

/**
 * Label fuer Schluessel.
 */
  private JLabel lbKey;

/**
 * Menuepunkt Codieren.
 */
  private JMenuItem miCodieren;

/* ------------------------------------------------- */
                                                 // MVC
/**
 * Mathematisches Modell,
 * enthaelt Funktionalitaet des XOR-Codierers.
 */
  private XORModel model;

/**
 * Zum View gehoeriger Cotroller.
 */
  private XORController controller;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor, setzt Ueberschrift und installiert MVC.
 * @param model Model, welches dargestellt werden soll
 * @param titel Ueberschrift in der Fenstertitelzeile
 */
  public XORView( XORModel model, String titel)
  {
// Titelzeile
    super( titel);
// Model
    this.model = model;
    this.model.addObserver( this);
// Controller
    controller = makeController();
// View
    makeView();
  }

/**
 * Erzeugt Controller,
 * Empfaenger fuer Ereignisse.
 * @return Controller fuer View
 */
  private XORController makeController()
  {
    return new XORController( model, this);
  }

/**
 * Erzeugt View,
 * baut die grafische Oberflaeche auf,
 * Controller verarbeitet Fensterereignisse.
 */
  private void makeView()
  {
// ContentPane
    getContentPane().add( createXORBox());

// Menue
    setJMenuBar( createMenuBar());

// Startwerte
    reset();
    model.reset();

// Fenster
    addWindowListener( controller);
    pack();
    setVisible( true);
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                         // ContentPane
/**
 * Senkrechte Box,
 * bestehend aus einer waagerechten Box zur Textanzeige
 * des originalen und des codierten Textes
 * und einem Schieberegister zur Schluesseleinstellung.
 */
  private Box createXORBox()
  {
    Box hbox = Box.createHorizontalBox();
    hbox.add( Box.createHorizontalStrut( 10));
    hbox.add( createTextAnzeige( true));    // original
    hbox.add( Box.createHorizontalStrut( 5));
    hbox.add( Box.createHorizontalStrut( 5));
    hbox.add( createTextAnzeige( false));    // codiert
    hbox.add( Box.createHorizontalStrut( 10));

    Box vbox = Box.createVerticalBox();
    vbox.add( hbox);                
    vbox.add( createSliderAnzeige());     // Schluessel

    return vbox;
  }

/**
 * Erzeugen eines Textfeldes.
 * @param istEingabe true, Textfeld fuer originalen Text
 *                   false, Textfeld fuer codierten Text
 */
  private Box createTextAnzeige( boolean istEingabe)
  {
    JTextArea feld = new JTextArea( 10, 40);
    JLabel label = new JLabel();

    if( istEingabe)
    {
      input = feld;
      lbInput = label;
    }
    else
    {
      output = feld;
      lbOutput = label;
    }

    feld.setEditable( false);
    feld.setLineWrap( true);

    JScrollPane pane = new JScrollPane( feld,
      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
      JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    Box box = Box.createVerticalBox();
    box.add( label);
    box.add( pane);
    box.add( Box.createVerticalStrut( 5));

    return box;
  }

/**
 * Erzeugen eines Schieberegisters
 * zur Schluesseleingabe,
 * Controller verarbeitet Registerereignisse.
 */
  private Box createSliderAnzeige()
  {
    JLabel label = new JLabel( "Eingabe Schlssel: ");

    slKey = new JSlider
      ( JSlider.HORIZONTAL, KEY_MIN, KEY_MAX, KEY_DEF);
    lbKey = new JLabel( "" + KEY_DEF);

    slKey.addChangeListener( controller);

    slKey.setMajorTickSpacing( 16);
    slKey.setMinorTickSpacing( 1);
    slKey.setPaintTicks( true);
    slKey.setPaintLabels( true);

    Box hbox = Box.createHorizontalBox();
    hbox.add( label);
    hbox.add( lbKey);

    Box box = Box.createVerticalBox();
    box.add( hbox);
    box.add( slKey);
    box.add( Box.createVerticalStrut( 5));

    return box;
  }

/**
 * Erzeugen eines Menues,
 * Controller verarbeitet Menueereignisse.
 */
  private JMenuBar createMenuBar()
  {
    JMenuBar  menuBar = new JMenuBar();

    JMenu menu = new JMenu( "Datei");;
    menu.setMnemonic( KeyEvent.VK_D);

    JMenuItem menuItem;

    menuItem = new JMenuItem( ACTION_NEU);
    menuItem.setMnemonic( ACTION_KEY_NEU);
    menuItem.addActionListener( controller);
    menu.add( menuItem);

    menuItem = new JMenuItem( ACTION_ORIGINAL);
    menuItem.setMnemonic( ACTION_KEY_ORIGINAL);
    menuItem.addActionListener( controller);
    menu.add( menuItem);

    menuItem = new JMenuItem( ACTION_CODIEREN);
    menuItem.setMnemonic( ACTION_KEY_CODIEREN);
    menuItem.addActionListener( controller);
    menu.add( menuItem);
    miCodieren = menuItem;

    menuItem = new JMenuItem( ACTION_SPEICHERN);
    menuItem.setMnemonic( ACTION_KEY_SPEICHERN);
    menuItem.addActionListener( controller);
    menu.add( menuItem);

    menuItem = new JMenuItem( ACTION_EXIT);
    menuItem.setMnemonic( ACTION_KEY_EXIT);
    menuItem.addActionListener( controller);
    menu.add( menuItem);

    menuBar.add( menu);

    return( menuBar);
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                    // service-Methoden fuer Controller
/**
 * Setzt Startzustand, Menuepunkt Neu.
 */
  public void reset()
  {
    setMiCodieren( false);
  }

/**
 * Setzen/Ruecksetzen des Menupunktes Codieren
 * und des schluessels.
 * @param enable true, falls Original codiert.
 */
  public void setMiCodieren( boolean enable)
  {
// Menupunkt Codieren
    miCodieren.setEnabled( enable);

// Schluessel
    lbKey.setText( "" + KEY_DEF);                  // 0
    slKey.setValue( KEY_DEF);

    lbKey.setEnabled( enable);
    slKey.setEnabled( enable);
  }

/**
 * Setzen des Label zum Schieberegister.
 * @param wert Wert des Schluessels
 */
  public void setLbKey( int wert)
  {
    lbKey.setText( "" + wert);
  }

/**
 * Lesen des vom Schieberegister eingestellten
 * Schluesselwertes.
 * @return aktueller Schluesselwert
 */
  public int getSlKey()
  {
    return slKey.getValue();
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                   // Observer-Methoden
/**
 * Ueberschreiben der Interfacemethode update,
 * legt Reaktion auf Aenderungen im Model fest.
 * @param m Model, welches Aenderungen meldet
 * @param o geaendertes Objekt
 */
  public void update( Observable m, Object o)
  {
    if( m == model)
    {
// Aktualisieren des Eingabefensters
      Code code = model.getGeladen();
      input.setText( getBytes( code));
      lbInput.setText
      ( "Eingabe: " + getDateiName( code));

// Aktualisieren des Ausgabefensters
// mit Schluesselangabe
      int schluessel
       = model.istCodiert()? getSlKey(): 0;

      code = model.getCodiert();
      output.setText( getBytes( code));
      lbOutput.setText
      ( "Ausgabe: " + getDateiName( code)
        + " Schlssel: " + schluessel);
    }
  }

/**
 * Lesen des Dateinamen eines Codes.
 * @return Dateiname als String
 */
  private String getDateiName( Code code)
  {
    String str = code.getDateiName();
    if( str == null) return "[kein Dateiname]";
    return str;
  }

/**
 * Darstellen eines Codes als String.
 * @return Code als String
 */
  private String getBytes( Code code)
  {
    return code.toString();
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Entfernt Fenster, deinstalliert MVC.
 */
  public void release()
  {
// Fenster
    dispose();
// Controller
    controller.release();
    controller = null;
// Model
    model.deleteObserver( this);
    model = null;
  }
}
